"""NilmDB

Object that represents a NILM database file"""

import tables
import time

class NilmDB(object):
    def __init__(self, filename):
        # Open or create the file
        self.h5file = tables.openFile(filename, "a", "NILM Database")

    def close(self):
        self.h5file.close()

    def stream_list(self):
        """Return list of paths to all Tables in the database"""
        iterator = self.h5file.walkNodes('/', 'Table')
        paths = [ x._v_pathname for x in iterator ]
        return sorted(paths)

    def stream_create(self, path, cls, index = None):
        """Create a table at the given path, with the contents
        matching the given class description (e.g. nilmdb.PrepData).
        Columns listed in 'index' are marked as indices.  If index =
        None, the 'timestamp' column is indexed if it exists.  Pass
        an empty list to prevent indexing"""
        [ group, node ] = path.rsplit("/", 1)
        if group == '':
            raise ValueError("Invalid path")

        # Make the group structure, one element at a time
        group_path = group.lstrip('/').split("/")
        for i in range(len(group_path)):
            parent = "/" + "/".join(group_path[0:i])
            child = group_path[i]
            try:
                self.h5file.createGroup(parent, child)
            except tables.NodeError:
                pass
        table = self.h5file.createTable(group, node, cls)

        # Create indices
        try:
            if index is None and "timestamp" in table.colnames:
                index = [ "timestamp" ]
            for ind in index:
                table.cols._f_col(ind).createIndex()
        except KeyError, e:
            # Remove this table if we got an error
            self.h5file.removeNode(group, node)
            raise e
            
