"""CherryPy-based server for accessing NILM database via HTTP"""

import nilmdb
import cherrypy
import sys

try:
    import cherrypy
    cherrypy.tools.json_out
except: # pragma: no cover
    sys.stderr.write("Cherrypy 3.2+ required\n")
    sys.exit(1)

class NilmApp(object):
    def __init__(self, db):
        self.db = db

class Root(NilmApp):
    """Root application for NILM database"""

    def __init__(self, db, version):
        super(Root, self).__init__(db)
        self.server_version = version

    @cherrypy.expose
    def index(self):
        raise cherrypy.NotFound()

    @cherrypy.expose
    def favicon_ico(self):
        raise cherrypy.NotFound()

    @cherrypy.expose
    @cherrypy.tools.json_out()
    def version(self):
        return self.server_version

class Stream(NilmApp):
    """Stream-specific operations"""

    @cherrypy.expose
    @cherrypy.tools.json_out()
    def list(self):
        return self.db.stream_list()

class Exiter(object):
    """App that exits the server, for testing"""
    @cherrypy.expose
    def index(self):
        cherrypy.response.headers['Content-Type'] = 'text/plain'
        def content():
            yield 'Exiting by request'
            raise SystemExit
        return content()
    index._cp_config = { 'response.stream': True }

class Server(object):
    version = "1.0"
    
    def __init__(self, filename, host = '127.0.0.1', port = 8080, stoppable = False):
        self.db = nilmdb.NilmDB(filename)
        cherrypy.config.update({
            'server.socket_host': host,
            'server.socket_port': port,
            'engine.autoreload_on': False,
            'environment': 'embedded',
            })
        cherrypy.tree.apps = {}
        cherrypy.tree.mount(Root(self.db, self.version), "/")
        cherrypy.tree.mount(Stream(self.db), "/stream")
        if stoppable:
            cherrypy.tree.mount(Exiter(), "/exit")

    def start(self, blocking = False, event = None):
        cherrypy.engine.start()
        if event is not None:
            event.set()
        if blocking:
            cherrypy.engine.wait(cherrypy.engine.states.EXITING,
                                 interval = 0.1, channel = 'main')

    def stop(self):
        cherrypy.engine.exit()
        self.db.close()
